package vavetam.api.structure;


import java.util.Set;

import vavetam.implementation.generation.incremental.formulation.Dictionary.Gender;
import vavetam.implementation.structure.enums.RefOType;

public interface IReferentialObject {
	
	/**
	 * @return a String that serves as a unique identifier of this {@link IReferentialObject}
	 */
	public String getIdentifier();
	
	/**
	 * @return the {@link RefOType} of this {@link IReferentialObject}
	 */
	public RefOType getType();
	
	/**
	 * @return a set of all {@link IReferentialObject}s which are assigned to be parents/superparts of this {@link IReferentialObject}.
	 */
	public Set<IReferentialObject> getParents();
	
	/**
	 * @return a set of all {@link IReferentialObject}s which are assigned to be children/subparts of this {@link IReferentialObject}.
	 */
	public Set<IReferentialObject> getChildren();
	
	/**
	 * @return a set of all {@link IDescriptor}s assigned to this {@link IReferentialObject}
	 */
	public Set<IDescriptor> getDescriptors();
	
	/**
	 * @return the definite identification of this {@link IReferentialObject} or <code>null</code> if none available
	 */
	public String getDefiniteIdentification();
	
	/**
	 * @return the gender of this {@link IReferentialObject} or <code>null</code> if none available
	 */
	public Gender getGender();
}