package vavetam.implementation.fileio;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;

import vavetam.implementation.simulation.ExplorationSituation;



/**
 * Instances of this class can be used to read in simulation schedules from cvs files.
 * 
 * @author Ole Eichhorn
 *
 */
public class CSVParser {
	
	Logger log = Logger.getLogger(CSVParser.class);
	
	public Set<ExplorationSituation> loadCSVschedule(File file) throws IOException
	{
		log.info("using " + file.getName() + " as schedule file to simulate input.");
		/*
		 *  prepare reading in the file in the given location and split it 
		 *  up into a set of single lines
		 */
		List<String> _lines = new ArrayList<String>();
		_lines.clear();
		FileReader reader = new FileReader(file);
		BufferedReader buffer = new BufferedReader(reader);
		while (buffer.ready()){
	       	String _string = buffer.readLine();
	       	_lines.add(_string);
	       }
		buffer.close();
		_lines.remove(0);
		
		/*
		 * now parse the lines, translate each into a single exploration
		 * situation and hand them back in the end
		 */
		Set<ExplorationSituation> result = new HashSet<ExplorationSituation>();
		for (String line:_lines)
		{
			StringTokenizer _cellparser = new StringTokenizer(line,",");
			int time = new Integer(_cellparser.nextToken()).intValue();
			if (!_cellparser.hasMoreElements()) {
				result.add(new ExplorationSituation(time));
				continue;
			}
			String type = removeChar('"',_cellparser.nextToken());
			String primaryFocus = removeChar('"',_cellparser.nextToken());
			String[] secFoc = null;
			boolean hasSecFoc = false;
			if (_cellparser.hasMoreTokens()) {
				hasSecFoc = true;
				String secondaryFocus = removeChar('"',_cellparser.nextToken());
				secFoc = parseSecFoc(secondaryFocus);
			}
			
			ExplorationSituation newSituation = hasSecFoc ? 
					new ExplorationSituation(time, type, primaryFocus, secFoc) :
					new ExplorationSituation(time, type, primaryFocus);
			
			log.trace("parsed " + newSituation);
			result.add(newSituation);
		}
		log.debug("finished parsing simulation schedule file");
		return result;
	}
	
	
	
	/**
	 * In case the MEP inside the exploration situation has secondary foci this
	 * method parses the string section representing them
	 * 
	 * @param s the string section representing the secondary foci
	 * @return an array of strings representing the secondary foci
	 */
	private String[] parseSecFoc(String s) {
		StringTokenizer st = new StringTokenizer(s,";");
		String[] sfs = new String[st.countTokens()];
		int i = 0;
		while (st.hasMoreElements()) {
			sfs[i] = st.nextToken();
			i++;
		}
		return sfs;
	}
	
	/**
	 * This convenience method deletes all occurrences of the specified 
	 * character c from the given string s and returns the resulting
	 * string.
	 * 
	 * @param c the character to be removed
	 * @param s the input string 
	 * @return the string with any characters c removed
	 */
	private String removeChar(char c, String s) {
		String out = "";
		for (int i = 0; i < s.length(); i++) {
			if (!(s.charAt(i) == c)) {
				out = out+s.charAt(i);
			}
		}
		return out;
	}
}
