package vavetam.implementation.generation.incremental;


import java.util.List;

import org.apache.log4j.Logger;

import vavetam.api.structure.IDescriptor;
import vavetam.api.structure.IMapExploratoryProcedure;
import vavetam.api.structure.IPreverbalMessage;
import vavetam.api.structure.IReferentialObject;
import vavetam.implementation.structure.PreverbalMessage;
import vavetam.implementation.structure.enums.Intention;
import vavetam.implementation.structure.util.PredicateUtils;


import com.google.common.base.Predicate;
import com.google.common.base.Predicates;
import com.google.common.collect.Collections2;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;

public class PreverbalMessageSelectionModule extends AbstractPipelineProcessor<IMapExploratoryProcedure, IMapExploratoryProcedure> {
	
	private Logger log = Logger.getLogger(PreverbalMessageSelectionModule.class);

	public PreverbalMessageSelectionModule(AbstractPipelineProcessor<? extends Object, IMapExploratoryProcedure> predecessor) {
		predecessor.registerListener(this);
		setDaemon();
		start();
	}
	
	@Override
	public IMapExploratoryProcedure process (IMapExploratoryProcedure mep) {
		List<IPreverbalMessage> result = Lists.newArrayList();
		log.trace("received " + mep.getUtterancePlan());
		for (Intention i: mep.getUtterancePlan().getIntentions()) {
			log.trace("for " + i + "...");
			switch (i) {
			case IDENTIFICATION : result.add(makeIdentificationMessage(mep.getUtterancePlan().getReferentialObject(), mep)); break;
			case SPATIAL_RELATIONS : result.add(makeSpatialRelationsMessage(mep.getUtterancePlan().getReferentialObject(), mep)); break;
			case EXTENSION : ; //TODO: finish me
			case JUNCTIONS : ;
			case GEOMETRIC_RELATIONS : ;
			default: log.trace("the preverbal message selection is not yet implemented!");
			}
		}
		List<IPreverbalMessage> finalResult = Lists.newArrayList(Collections2.filter(result, Predicates.notNull()));
		mep.setPreverbalMessages(finalResult);
		return mep;
	}
	
	private IPreverbalMessage makeSpatialRelationsMessage(IReferentialObject referentialObject, IMapExploratoryProcedure mep) {
		log.trace("...we make a message about spatial relations");
		IPreverbalMessage pvm = new PreverbalMessage(mep);
		List<IDescriptor> descriptors = Lists.newArrayList(Sets.filter(referentialObject.getDescriptors(), new Predicate<IDescriptor>() {
			@Override
			public boolean apply(IDescriptor descriptor) {
				return PredicateUtils.isSpatialRelationPredicate(descriptor.getPredicateAtIndex(0));
			}
		}));
		log.trace("selected " + descriptors.size() + " descriptors");
		if (descriptors.size() > 0) {
			pvm.setDescriptors(descriptors);
			pvm.setReferentialObject(referentialObject);
			pvm.setIntention(Intention.SPATIAL_RELATIONS);
			return pvm;
		} else {
			return null;
		}
	}

	private IPreverbalMessage makeIdentificationMessage(IReferentialObject referentialObject, IMapExploratoryProcedure mep) {
		log.trace("...we make a message to identify its referential object");
		IPreverbalMessage pvm = new PreverbalMessage(mep);
		if (referentialObject.getDefiniteIdentification() == null) {
			List<IDescriptor> descriptors = Lists.newArrayList(Sets.filter(referentialObject.getDescriptors(), new Predicate<IDescriptor>() {
				@Override
				public boolean apply(IDescriptor descriptor) {
					return PredicateUtils.isIndefiniteIdentificationPredicate(descriptor.getPredicateAtIndex(0));
				}
			}));
			pvm.setDescriptors(descriptors);
			log.trace("selected " + descriptors.size() + " descriptors");
		}
		log.trace("selected available definite identification " + referentialObject.getDefiniteIdentification());
		pvm.setReferentialObject(referentialObject);
		pvm.setIntention(Intention.IDENTIFICATION);
		pvm.setRepetition(mep.getUtterancePlan().isRepetition());
		return pvm;
	}

	@Override
	protected String threadName() {
		return "Preverbal Message Selector";
	}

	@Override
	protected Logger getLogger() {
		return log;
	}
}
