package vavetam.implementation.generation.incremental;


import org.apache.log4j.Logger;

import vavetam.api.structure.IMapExploratoryProcedure;
import vavetam.implementation.generation.incremental.History.Difference;
import vavetam.implementation.structure.MapExploratoryProcedure;
import vavetam.implementation.structure.UtterancePlan;
import vavetam.implementation.structure.enums.MEPType;


public class UtterancePlanSelectorModule extends AbstractPipelineProcessor<MapExploratoryProcedure, MapExploratoryProcedure> {
	
	private Logger log = Logger.getLogger(UtterancePlanSelectorModule.class);
	private History history = new History();
	
	public UtterancePlanSelectorModule(AbstractPipelineProcessor<? extends Object, MapExploratoryProcedure> predecessor) {
		predecessor.registerListener(this);
		setDaemon();
		start();
	}
	
	@Override
	public MapExploratoryProcedure process(MapExploratoryProcedure mep) {
		if (mep.getType() == MEPType.STOP) {
			// TODO: implement stops
			return null;
		} else {
			UtterancePlan utterancePlan = getUtterancePlan(mep);
			log.debug("selected " + utterancePlan + " for " + mep);
			history.memorize(mep, utterancePlan);
			mep.setUtterancePlan(utterancePlan);
			return mep;
		}
	}
	
	private UtterancePlan getUtterancePlan(IMapExploratoryProcedure mep) {
		Difference d = history.calculateDifferenceToLast(mep);
		switch (d.getType()) {
		case PRIMARY_FOCUS_CHANGE: { 
			UtterancePlan iu = new UtterancePlan(mep.getPrimaryFocus());
			boolean hadThat = history.hadThat(mep, iu);
			iu.setRepetition(hadThat);
			return iu;
		}
		//TODO: not only primary focus...
		case SECONDARY_FOCUS_ADDED:
		case SECONDARY_FOCUS_REMOVED:
		case SECONDARY_FOCUS_ADDED_AND_REMOVED:
		case NO_CHANGE: 
		}
		return new UtterancePlan();
	}

	@Override
	protected String threadName() {
		return "Utterance Plan Selector";
	}

	@Override
	protected Logger getLogger() {
		return log;
	}
}