package vavetam.implementation.generation.incremental.formulation;


import java.text.MessageFormat;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import vavetam.api.structure.IFormulation;
import vavetam.api.structure.IPreverbalMessage;
import vavetam.implementation.structure.Formulation;
import vavetam.implementation.structure.enums.Intention;


import static java.text.MessageFormat.format;

public class IdentificationFormulator implements IFormulator {
	
	private final Logger log = Logger.getLogger(IdentificationFormulator.class);

	@Override
	public List<IFormulation> formulate(IPreverbalMessage... pvms) {
		if (pvms.length == 1) {
			IPreverbalMessage pvm = pvms[0];
			List<IFormulation> result = new LinkedList<IFormulation>();
			for (String s: Dictionary.createIdentificationMessage(pvm)) {
				result.add(new Formulation(s, pvm));
			}
			return result;
		} else if (pvms.length == 2) {
			if (Intention.IDENTIFICATION.equals(pvms[0].getIntention()) && Intention.IDENTIFICATION.equals(pvms[1].getIntention())) {
				/*
				 * Fälle: 
				 * 
				 * markierung xxx ist da, wo der satz zum vorgänger unterschiedlich wird
				 * daraus folgt -> zugunsten der variation wird immer versucht den UNähnlichsten satz zu sagen.
				 * je weiter der Satz fortgeschritten ist, desto weniger möglichkeiten zur anpassung bestehen, so werden alle mal verewndet
				 * 
				 * (Ausgangssatz: Das ist die Blumenstraße.) 
				 * 
				 * Das ist der Amselweg; [covert repair]
				 *         xxx
				 * 
				 * Das ist die ... ääh ... der Amselweg; [overt repair]
				 *             xxx
				 *             
				 * Das war die Blumenstraße und jetzt bist du auf dem Amselweg;
				 *     xxx
				 * 
				 * Das ist die Blumenstraße ääh der Amselweg; 
				 *                          xxx
				 * 
				 * Das ist die Blumenstraße gewesen und jetzt bist du auf dem Amselweg;
				 *                          xxxxxxx
				 *                          
				 * die unteren drei Möglichkeiten gibt es für alle Variationen von Anfangssätzen, die ich habe, also:
				 * 
				 * * dies ist, 
				 * * das ist, 
				 * * hier ist,
				 * * und jetzt bist du auf...
				 * 
				 * außerdem gibt es von jedem dieser Sätze eine version mit einem eingebauten "wieder".
				 * 
				 * auf all diese soll durch anpassung reagiert werden können.
				 * zunächst werden einfach alle ausmodelliert
				 */
				List<String> strings = new LinkedList<String>();
				log.trace(format("formulating identification for {0} and {1}", pvms[0], pvms[1]));
				strings.addAll(Dictionary.createIdentificationMessage(pvms[1])); //covert repairs
				strings.addAll(Dictionary.createCompoundIdentificationMessage(pvms[0], pvms[1]));
				List<IFormulation> result = new LinkedList<IFormulation>();
				for (String s: strings) {
					result.add(new Formulation(s, pvms));
				}
				return result;
			} else {
				log.debug("sorry, so far I can only handle multiple PVMs if they all have the intention \"IDENTIFICATION\"");
			}
		} else {
			log.debug(MessageFormat.format("can't formulate [{}], wrong number of preverbal messages", StringUtils.join(pvms, ",")));
		}
		return Collections.emptyList();
	}
}
