package vavetam.implementation.generation.incremental.formulation;

import static vavetam.implementation.generation.incremental.formulation.Dictionary.AND;
import static vavetam.implementation.generation.incremental.formulation.Dictionary.ARE;
import static vavetam.implementation.generation.incremental.formulation.Dictionary.HESITATION;
import static vavetam.implementation.generation.incremental.formulation.Dictionary.IS;
import static vavetam.implementation.generation.incremental.formulation.Dictionary.PERIOD;
import static vavetam.implementation.generation.incremental.formulation.Dictionary.WHITESPACE;
import static vavetam.implementation.generation.incremental.formulation.Dictionary.formConcatenationSentence;
import static vavetam.implementation.generation.incremental.formulation.Dictionary.getOrientation2;
import static vavetam.implementation.generation.incremental.formulation.Dictionary.getPersPronom;
import static vavetam.implementation.generation.incremental.formulation.Dictionary.identify;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;

import org.apache.commons.lang.StringUtils;

import vavetam.api.structure.IDescriptor;
import vavetam.api.structure.IFormulation;
import vavetam.api.structure.IPredicate;
import vavetam.api.structure.IPreverbalMessage;
import vavetam.api.structure.IReferentialObject;
import vavetam.implementation.generation.incremental.formulation.Dictionary.Casus;
import vavetam.implementation.structure.Formulation;
import vavetam.implementation.structure.enums.Intention;
import vavetam.implementation.structure.enums.PredicateName;


import com.google.common.collect.LinkedListMultimap;
import com.google.common.collect.Lists;
import com.google.common.collect.Multimap;

public class SpatialRelationsFormulator implements IFormulator{

	@Override
	public List<IFormulation> formulate(IPreverbalMessage... pvms) {
		if (pvms.length == 1) {
			String result = formulateInternal(pvms[0]);
			IFormulation formulation = new Formulation(result, pvms[0]);
			return Collections.singletonList(formulation);
		} else if (pvms.length == 2) {
			// TODO: das gehört eindeutig eher in den IdentificationFormulator und würde da jetzt auch genau den gleichen job erledigen, nur dass man automatisch ALLE fälle abfangen könnte. also für ALLE möglichen vorhergehenden Intentions!
			// fall: beim beschreiben der spatial relations wird focus gewechselt und vavetam fängt wieder bei identification an
			if (Intention.IDENTIFICATION.equals(pvms[1].getIntention())) {
				List<String> stringResults = Lists.newArrayList();
				
				//erstmal den 2. satz formulieren
				List<IFormulation> secondSentenceFormulations = (new IdentificationFormulator()).formulate(pvms[1]);
				
				StringBuilder stepwiseBuilder = new StringBuilder();
				String firstSentence = formulateInternal(pvms[0]);
				firstSentence = StringUtils.chop(firstSentence);
				for (String word : firstSentence.split(" ")) {
					stepwiseBuilder.append(word).append(WHITESPACE);
					String finalSentence = new StringBuilder()
						.append(stepwiseBuilder.toString())
						.append(HESITATION)
						.append(WHITESPACE)
						.append("also ")
						.toString();
					for (IFormulation formulation : secondSentenceFormulations) {
						stringResults.add(finalSentence + StringUtils.uncapitalize(formulation.getText()));
					}
					
				}
				List<IFormulation> result = Lists.newArrayList();
				for (String sentence: stringResults) {
					result.add(new Formulation(sentence, pvms));
				}
				return result;
			}
		}
		return Collections.emptyList();
	}

	private String formulateInternal(IPreverbalMessage pvm) {
		// sort ReferentialObjects into spatial directions to formulate summarizing sentences

		Multimap<PredicateName, IReferentialObject> spatialRelations = LinkedListMultimap.create();
		for (IDescriptor descriptor: pvm.getDescriptors()) {
			// TODO: up to now, only the first predicate of every descriptor is being considered...
			IPredicate predicate = descriptor.getPredicateAtIndex(0);
			// we only use one direction of stating a spatial relation
			if (pvm.getReferentialObject().equals(predicate.getArguments().get(1))) {
				spatialRelations.put(predicate.getName(), predicate.getArguments().get(0));
			}
		}
		// sentences are being prepared: key strings are being selected and put into arrays, one per spatial direction
		List<String[]> subresults = new LinkedList<String[]>();
		for (Entry<PredicateName, Collection<IReferentialObject>> e: spatialRelations.asMap().entrySet()) {
			List<String> objects = new LinkedList<String>();
			for (IReferentialObject refo: e.getValue()) {
				objects.add(identify(refo, Casus.NOMINATIV));
			}
			String[] subresult = new String[3];
			subresult[0] = getOrientation2(e.getKey());
			subresult[1] = objects.size() < 2 ? IS : ARE;
			subresult[2] = formConcatenationSentence(objects);
			subresults.add(subresult);
		}
		// sentences get completely formulated now
		StringBuilder result = new StringBuilder();
		String persPronom = getPersPronom(Casus.DATIV, pvm.getReferentialObject());
		if (subresults.size() == 1) {
			String[] msg = subresults.get(0);
			result.append(StringUtils.capitalize(msg[0])) //Über
			.append(WHITESPACE)
			.append(persPronom) //ihr
			.append(WHITESPACE)
			.append(msg[1]) // ist
			.append(WHITESPACE)
			.append(msg[2]) // das Rathaus
			.append(PERIOD);
		} else if (subresults.size() == 2) {
			String[] msg1 = subresults.get(0);
			String[] msg2 = subresults.get(1);
			result.append(StringUtils.capitalize(msg1[0])) //Links neben
			.append(WHITESPACE)
			.append(persPronom) //ihm
			.append(WHITESPACE)
			.append(msg1[1]) //sind
			.append(WHITESPACE)
			.append(msg1[2]) //die Gedächniskirche und der Lidl
			.append(WHITESPACE)
			.append(AND) //und
			.append(WHITESPACE)
			.append(msg2[0]) //über
			.append(WHITESPACE)
			.append(persPronom) //ihm
			.append(WHITESPACE)
			.append(msg2[2]) //die Anne-Frank-Schule
			.append(PERIOD);
		}
		return result.toString();
	}
}
