package vavetam.implementation.simulation;

import static java.text.MessageFormat.format;
import inpro.incremental.processor.SynthesisModule;
import inpro.incremental.unit.EditMessage;
import inpro.incremental.unit.EditType;
import inpro.incremental.unit.IU;
import inpro.incremental.unit.IU.Progress;
import inpro.incremental.unit.PhraseIU;

import java.util.Collection;
import java.util.List;
import java.util.concurrent.ConcurrentLinkedQueue;

import org.apache.log4j.Logger;

import vavetam.implementation.generation.incremental.heuristic.SimpleSpeechDurationHeuristic;
import vavetam.implementation.generation.incremental.heuristic.SpeechDurationHeuristic;

import com.google.common.collect.Queues;

public class DummySpeechEngine extends SynthesisModule implements Runnable {
	
	private Logger log = Logger.getLogger(DummySpeechEngine.class);
	private ConcurrentLinkedQueue<IU> currentIUs = Queues.newConcurrentLinkedQueue();
	private Thread sayer = new Thread(this);
	private boolean sayerRunning = false;
	private boolean finish = false;
	private SpeechDurationHeuristic heuristic = new SimpleSpeechDurationHeuristic(50);
	
	public DummySpeechEngine() {
		sayer.setName("Dummy Speech Engine");
		sayer.start();
	}

	@Override
	public void hypChange(Collection<? extends IU> ius,
			List<? extends EditMessage<? extends IU>> edits) {
		currentIUs.addAll(Queues.newConcurrentLinkedQueue(ius));
		for (EditMessage<? extends IU> edit: edits ) {
			if (EditType.REVOKE.equals(edit.getType())) {
				IU revokeIU = edit.getIU();
				currentIUs.remove(revokeIU);
			}
		}
		rightBuffer.setBuffer(ius, edits);
		synchronized (this) {
			notify();
		} 
	}
	
	public void finish() {
		finish = true;
		synchronized(this) {
			notify();
		}
	}
	
	@Override
	public void run() {
		sayerRunning = true;
		while (sayerRunning) {
			if (currentIUs.isEmpty()) {
				if (finish) {
					sayerRunning = false;
					continue;
				} else {
					try {
						synchronized(this) {
							wait();
							continue;
						}
					} catch (Exception e) {continue;}
				}
			}
			while (!currentIUs.isEmpty()) {
				IU iu = currentIUs.poll();
				try {
					PhraseIU piu = (PhraseIU) iu;
					int duration = heuristic.getDurationInMillis(piu.toPayLoad());
					log.info(format("\"`{1}\"'' (takes {0}ms to say)",duration,iu.toPayLoad()));
					if (piu instanceof PhraseIUMock)
						((PhraseIUMock) piu).setProgress(Progress.ONGOING); 
					piu.notifyListeners();
					Thread.sleep(duration);
					if (piu instanceof PhraseIUMock)
						((PhraseIUMock) piu).setProgress(Progress.COMPLETED);
					piu.notifyListeners();
				} catch (InterruptedException e) {
					log.trace("couldn't sleep that long");
				}
			}
		}
	}
}
