package vavetam.implementation.simulation;

import org.apache.commons.lang.StringUtils;

/**
 * an ExplorationSituation contains information needed for simulated input.
 * That is an MEP and a time it is supposed to be fired.
 * 
 * @author Ole Eichhorn
 *
 */
public class ExplorationSituation {

	private int msec;
	private boolean done = false;
	private String type, pfocid;
	private String[] sfocid;
	private boolean nomep = false;
	
	public ExplorationSituation(int msec, String type, String pfocid, String... sfocid) {
		this.msec = msec;
		this.type = type;
		this.pfocid = pfocid;
		this.sfocid = sfocid;
	}
	
	/**
	 * ExplorationSituations only specified with a time resemble a StopMEP
	 * @param msec
	 */
	public ExplorationSituation(int msec) {
		this.msec = msec;
		nomep = true;
	}

	/**
	 * @return the time, when this ExplorationSituation is supposed to be
	 * 			fired.
	 */
	public int getMilliseconds() {
		return msec;
	}

	/**
	 * @return the MEP type
	 */
	public String getType() {
		return type;
	}

	/**
	 * @return the primary focus of the MEP
	 */
	public String getPrimaryFocusID() {
		return pfocid;
	}

	/**
	 * @return the secondary foci of the MEP
	 */
	public String[] getSecondaryFocusIDs() {
		return sfocid;
	}
	
	/**
	 * indicates that this ExplorationSituation has been processed
	 */
	public void setDone() {
		done = true;
	}
	
	/**
	 * @return true, if this ExplorationSituation has already been processed
	 */
	public boolean isDone() {
		return done;
	}
	
	public boolean isNoMEP() {
		return nomep;
	}
	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("ExplorationSituation[");
		sb.append("@" + getMilliseconds() + "ms, ");
		if (nomep) {
			sb.append("STOP]");
		} else {
			sb.append(getType() + ", ");
			sb.append("primary focus: " + getPrimaryFocusID());
			if (sfocid.length > 0) {
				sb.append(", secondary foci: {" + StringUtils.join(sfocid, ", ") + "}");
			}
			sb.append("]");
		}
		return sb.toString();
	}
}
