package vavetam.implementation.structure;

import inpro.incremental.unit.PhraseIU;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang.StringUtils;

import vavetam.api.structure.IFormulation;
import vavetam.api.structure.IMapExploratoryProcedure;
import vavetam.api.structure.IPreverbalMessage;
import vavetam.implementation.simulation.PhraseIUMock;


import com.google.common.collect.Sets;

public class Formulation implements IFormulation {
	
//	private Logger log = Logger.getLogger(Formulation.class);
	
	private static boolean DUMMY_MODE = false;
	
	private List<PhraseIU> ius = new ArrayList<PhraseIU>();
	private String text;
	private Set<IPreverbalMessage> pvms;
	private Set<IMapExploratoryProcedure> meps;
	
	public Formulation(String text, IPreverbalMessage... pvms) {
		this.text = text;
		this.pvms = Sets.newHashSet(pvms);
		
		this.meps = new HashSet<IMapExploratoryProcedure>();
		for (IPreverbalMessage pvm: pvms) {
			meps.add(pvm.getMep());
		}
		this.ius = toIUs(text);
	}
	
	private List<PhraseIU> toIUs(String phrase) {
		if (StringUtils.isBlank(phrase)) {
			return Collections.emptyList();
		}
		String[] words = phrase.split("\\s+");
		List<PhraseIU> result = new ArrayList<PhraseIU>();
		for (String word : words) {
			if (DUMMY_MODE) {
				result.add(new PhraseIUMock(word));
			} else {
				result.add(new PhraseIU(word));
			}
		}
		result.get(result.size()-1).setFinal();
		return result;
	}
	
	/* (non-Javadoc)
	 * @see implementation.structure.IFormulation#getIus()
	 */
	@Override
	public List<PhraseIU> getIus() {
		return ius;
	}
	
	/* (non-Javadoc)
	 * @see implementation.structure.IFormulation#getText()
	 */
	@Override
	public String getText() {
		return text;
	}
	
	/* (non-Javadoc)
	 * @see implementation.structure.IFormulation#getPvms()
	 */
	@Override
	public Set<IPreverbalMessage> getPvms() {
		return pvms;
	}
	
	/* (non-Javadoc)
	 * @see implementation.structure.IFormulation#getMeps()
	 */
	@Override
	public Set<IMapExploratoryProcedure> getMeps() {
		return meps;
	}
	
	public String toString() {
		return "\"`" + text + "\"'";
	}

	@Override
	public boolean isCompleted() {
		// TODO Auto-generated method stub
		return ius.get(ius.size()-1).isCompleted();
	}
	
	public static void setDummyMode(boolean mode) {
		DUMMY_MODE = mode;
	}
}
