package vavetam.implementation.structure;

import inpro.incremental.unit.PhraseIU;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;

import vavetam.api.structure.IFormulation;
import vavetam.api.structure.IMapExploratoryProcedure;
import vavetam.api.structure.IPreverbalMessage;
import vavetam.api.structure.IReferentialObject;
import vavetam.api.structure.IUtterancePlan;
import vavetam.implementation.structure.enums.MEPType;


public class MapExploratoryProcedure implements IMapExploratoryProcedure {
	
	private MEPType type;
	private IReferentialObject primaryFocus;
	private List<IReferentialObject> secondaryFocus;
	private IUtterancePlan utterancePlan;
	private List<IPreverbalMessage> pvms;
	private Map<IPreverbalMessage, List<IFormulation>> formulations = new HashMap<IPreverbalMessage, List<IFormulation>>();
	
	private boolean hasUtterancePlan = false;
	private boolean hasPvms = false;
	private boolean hasFormulations = false;

	public MapExploratoryProcedure(MEPType type, IReferentialObject primaryFocus, List<IReferentialObject> secondaryFoci) {
		this.type = type;
		this.primaryFocus = primaryFocus;
		this.secondaryFocus = secondaryFoci;
	}
	
	public MapExploratoryProcedure(MEPType type, IReferentialObject primaryFocus) {
		this.type = type;
		this.primaryFocus = primaryFocus;
	}
	
	public MapExploratoryProcedure() {
		this.type = MEPType.STOP;
		this.secondaryFocus = Collections.emptyList();
	}

	@Override
	public IReferentialObject getPrimaryFocus() {
		return primaryFocus;
	}

	@Override
	public List<IReferentialObject> getSecondaryFocus() {
		return secondaryFocus;
	}

	@Override
	public MEPType getType() {
		return type;
	}

	@Override
	public boolean primaryFocusEquals(IMapExploratoryProcedure mep) {
		return this.getPrimaryFocus().equals(mep.getPrimaryFocus());
	}

	@Override
	public List<IReferentialObject> secondaryFocusDifferences(
			IMapExploratoryProcedure m) {
		// TODO Auto-generated method stub
		return null;
	}

//	public String toPayLoad() {
//		return toString();
//	}
	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("MapExploratoryProcedure[");
		sb.append(type + ", ");
		sb.append(primaryFocus);
		if (!secondaryFocus.isEmpty()) {
			sb.append(", {" + StringUtils.join(secondaryFocus, ", ") + "}");
		}
		sb.append("]");
		return sb.toString();
	}
	
	public void setUtterancePlan(IUtterancePlan utterancePlan) {
		this.utterancePlan = utterancePlan;
		hasUtterancePlan = true;
	}

	public IUtterancePlan getUtterancePlan() {
		return utterancePlan;
	}
	
	public List<IPreverbalMessage> getPreverbalMessages() {
		return pvms;
	}
	
	public void setPreverbalMessages(List<IPreverbalMessage> pvms) {
		this.pvms = pvms;
		hasPvms = true;
	}

//	public List<Formulation> getFormulations(IPreverbalMessage pvm) {
//		return formulations.get(pvm);
//	}
//
//	public void putFormulation(PreverbalMessage pvm, List<Formulation> messages) {
//		formulations.put(pvm, messages);
//	}

	public boolean hasUtterancePlan() {
		return hasUtterancePlan;
	}

	public boolean hasPvms() {
		return hasPvms;
	}

	public boolean hasFormulations() {
		return hasFormulations;
	}
	
	public boolean isDone() {
		if (hasFormulations) {
			for (Entry<IPreverbalMessage, List<IFormulation>> e: formulations.entrySet()) {
				List <PhraseIU> ius = e.getValue().get(0).getIus();
				for (PhraseIU iu : ius) {
					if(!iu.isCompleted()) {
						return false;
					}
				}
			}
			return true;
		} else {
			return false;
		}
	}

	@Override
	public boolean isCompleted() {
		for (List<IFormulation> formulationList: formulations.values()) {
			if (!CollectionUtils.exists(formulationList, new org.apache.commons.collections.Predicate(){

				@Override
				public boolean evaluate(Object object) {
					return ((IFormulation) object).isCompleted();
				}
				
			})) {
				return false;
			}
		}
		return true;
	}
}
