package vavetam.implementation.structure;

import static vavetam.implementation.structure.enums.Intention.EXTENSION;
import static vavetam.implementation.structure.enums.Intention.GEOMETRIC_RELATIONS;
import static vavetam.implementation.structure.enums.Intention.IDENTIFICATION;
import static vavetam.implementation.structure.enums.Intention.JUNCTIONS;
import static vavetam.implementation.structure.enums.Intention.SPATIAL_RELATIONS;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import vavetam.api.structure.IReferentialObject;
import vavetam.api.structure.IUtterancePlan;
import vavetam.implementation.structure.enums.Intention;


public class UtterancePlan implements IUtterancePlan {
	
	private List<Intention> intentions = new ArrayList<Intention>();
	private boolean repetition;
	private IReferentialObject referentialObject;
	
	public UtterancePlan(IReferentialObject referentialObject) {
		// TODO different utterance plans for different classes of RefOs
		intentions.add(IDENTIFICATION);
		intentions.add(SPATIAL_RELATIONS);
		intentions.add(EXTENSION);
		intentions.add(JUNCTIONS);
		intentions.add(GEOMETRIC_RELATIONS);
		this.referentialObject = referentialObject;
	}
	
	public UtterancePlan(Intention... intentions) {
		this.intentions = Arrays.asList(intentions);
	}
	
	/* (non-Javadoc)
	 * @see implementation.structure.IUtterancePlan#getIntentions()
	 */
	@Override
	public List<Intention> getIntentions() {
		return intentions;
	}
	
	/* (non-Javadoc)
	 * @see implementation.structure.IUtterancePlan#getReferentialObject()
	 */
	@Override
	public IReferentialObject getReferentialObject() {
		return referentialObject;
	}
	
	/* (non-Javadoc)
	 * @see implementation.structure.IUtterancePlan#setRepetition(boolean)
	 */
	@Override
	public void setRepetition(boolean repetition) {
		this.repetition = repetition;
	}
	
	/* (non-Javadoc)
	 * @see implementation.structure.IUtterancePlan#isRepetition()
	 */
	@Override
	public boolean isRepetition() {
		return repetition;
	}
	
	@Override
	public String toString() {
		return "UtterancePlan [" + StringUtils.join(intentions, ", ") + "]";		
	}
	
	@Override
	public boolean equals(Object o) {
		if (o instanceof UtterancePlan) {
			IUtterancePlan other = (IUtterancePlan) o;
			boolean refOEquals = false;
			if (this.getReferentialObject() == null) {
				if (other.getReferentialObject() == null) {
					refOEquals = true;
				}
			} else if (other.getReferentialObject() != null) {
				refOEquals = this.getReferentialObject().equals(other.getReferentialObject());
			}
			return (this.getIntentions().equals(other.getIntentions()) && refOEquals);
		} else {
			return false;
		}
	}
	
	@Override
	public int hashCode() {
		int refOhash;
		if (getReferentialObject() != null) {
			refOhash = getReferentialObject().hashCode();
		} else {
			refOhash = 1;
		}
		return (getIntentions().hashCode() * refOhash);
	}
}
