package vavetam.implementation.structure.enums;

import java.util.EnumSet;

/**
 * This enum class defines the known predicates that can be assigned to a RefO.
 * 
 * @author Ole Eichhorn
 *
 */
public enum PredicateName {
	//nominal predicates
	BUILDING(1),
	CHURCH(1),
	CORNER(1),
	CROSSING(1),
	T_CROSSING(1),
	DEAD_END(1),
	BOTTOM_FRAME(1),
	LEFT_FRAME(1),
	RIGHT_FRAME(1),
	TOP_FRAME(1),
	MAP_FRAME(1),
	PARK(1),
	REGION(1),
	SCHOOL_BUILDING(1),
	SQUARE(1),
	STREET(1),
	TOWN_HALL(1),
	TREE(1),
	MAP(1),
	
	//spatial predicates
	///miscellaneous predicates
	IS_BETWEEN(3),
	IS_IN_TRACK_CONFIG(2),
	IS_LOCATED_IN_NREPR(2),
	IS_LOCATED_IN_REPR(2),
	IS_PARALLEL_TO(2),
	///has_x_limit predicates
	HAS_LEFT_LIMIT(2),
	HAS_RIGHT_LIMIT(2),
	HAS_UPPER_LIMIT(2),
	HAS_LOWER_LIMIT(2),
	HAS_UPPER_LEFT_LIMIT(2),
	HAS_UPPER_RIGHT_LIMIT(2),
	HAS_LOWER_LEFT_LIMIT(2),
	HAS_LOWER_RIGHT_LIMIT(2),
	///is_extreme predicates
	IS_LEFTMOST(1),
	IS_RIGHTMOST(1),
	IS_TOPMOST(1),
	IS_BOTTOMMOST(1),
	///is_located predicates
	IS_LEFT(2),
	IS_RIGHT(2),
	IS_ABOVE(2),
	IS_BELOW(2),
	IS_LEFT_ABOVE(2),
	IS_RIGHT_ABOVE(2),
	IS_LEFT_BELOW(2),
	IS_RIGHT_BELOW(2),
	///is_part predicates
	IS_MIDDLE_PART(2),
	IS_LEFT_PART(2),
	IS_RIGHT_PART(2),
	IS_UPPER_PART(2),
	IS_LOWER_PART(2),
	IS_UPPER_LEFT_PART(2),
	IS_UPPER_RIGHT_PART(2),
	IS_LOWER_LEFT_PART(2),
	IS_LOWER_RIGHT_PART(2),
	
	///other predicates
	IS_PART_OF(2);
	
	private int arity;
	
	PredicateName(int arity) {
		this.arity = arity;
	}
	
	public int getArity() {
		return arity;
	}
	
	
	private static final EnumSet<PredicateName> LIMIT_PREDICATES = EnumSet.of (
			HAS_LEFT_LIMIT, HAS_LOWER_LEFT_LIMIT, HAS_LOWER_LIMIT, HAS_LOWER_RIGHT_LIMIT,
			HAS_RIGHT_LIMIT, HAS_UPPER_LEFT_LIMIT, HAS_UPPER_LIMIT, HAS_UPPER_RIGHT_LIMIT);
	
	private static final EnumSet<PredicateName> PART_OF_PREDICATES = EnumSet.of (
			IS_LEFT_PART, IS_LOWER_LEFT_PART, IS_LOWER_PART, IS_LOWER_RIGHT_PART,
			IS_RIGHT_PART, IS_UPPER_LEFT_PART, IS_UPPER_PART, IS_UPPER_RIGHT_PART,
			IS_MIDDLE_PART);
	
	private static final EnumSet<PredicateName> INDEFINITE_IDENTIFICATION_PREDICATES = EnumSet.of (
			STREET, BUILDING, MAP_FRAME, DEAD_END, SQUARE, PARK, REGION, SCHOOL_BUILDING,
			TOWN_HALL, CHURCH, TREE, CROSSING, T_CROSSING, CORNER, MAP);
	
	private static final EnumSet<PredicateName> SPATIAL_RELATION_PREDICATES = EnumSet.of (
			IS_ABOVE, IS_BELOW, IS_BETWEEN, IS_LEFT, IS_LEFT_ABOVE, IS_LEFT_BELOW,
			IS_RIGHT, IS_RIGHT_ABOVE, IS_RIGHT_BELOW);
	
	public static boolean isLimitPredicate(PredicateName name) {
		return LIMIT_PREDICATES.contains(name);
	}
	
	public static boolean isPartOfPredicate(PredicateName name) {
		return PART_OF_PREDICATES.contains(name);
	}
	
	public static boolean isIndefiniteIdentificationPredicate(PredicateName name) {
		return INDEFINITE_IDENTIFICATION_PREDICATES.contains(name);
	}
	
	public static boolean isSpatialRelationPredicate(PredicateName name) {
		return SPATIAL_RELATION_PREDICATES.contains(name);
	}
}
