package vavetam.implementation.structure.refo;

import org.apache.commons.lang3.StringUtils;

import vavetam.api.structure.IDescriptor;
import vavetam.api.structure.IReferentialObject;
import vavetam.implementation.generation.incremental.formulation.Dictionary.Gender;
import vavetam.implementation.structure.enums.RefOType;


public class ReferentialObjectBuilder {
	
	private ReferentialObjectImpl result = null; 
	
	public ReferentialObjectBuilder(String identifier, RefOType type) {
		result = new ReferentialObjectImpl(identifier, type);
	}
	
	public void addParent(IReferentialObject parent) {
		result.addParent(parent);
	}
	
	public void addChild(IReferentialObject child) {
		result.addChild(child);
	}
	
	public void setDefiniteIdentification(String identification) {
		result.setDefiniteIdentification(identification);
	}
	
	public void addDescriptor(IDescriptor d) {
		result.addDescriptor(d);
	}
	
	public void setGender(Gender gender) {
		result.setGender(gender);
	}
	
	public IReferentialObject result() {
		return result;
	}
	
	public String getIdentifier() {
		return result.getIdentifier();
	}
	
	@Override
	public String toString() {
		return "Builder for " + result;
	}
	
	public String descriptorsToString() {
		return "[" + StringUtils.join(result.getDescriptors(), ", ") + "]";
	}
	
	public String parentsToString() {
		return "[" + StringUtils.join(result.getParents(), ", ") + "]";
	}
	
	public String childrenToString() {
		return "[" + StringUtils.join(result.getChildren(), ", ") + "]";
	}
	
	public String definiteIdentificationAndGenderToString() {
		String name = result.getDefiniteIdentification() == null ? "" : result.getDefiniteIdentification();
		String gender = result.getDefiniteIdentification() == null ? "" : result.getGender().toString();
		String result = gender + " " + name;
		return StringUtils.isNotBlank(result) ? result : "no definite identification";
	}
	
	public String toVerboseString() {
		StringBuilder sb = new StringBuilder();
		sb.append("\n" + toString() + "\n");
		sb.append(definiteIdentificationAndGenderToString() == null ? "" : definiteIdentificationAndGenderToString() + "\n");
		sb.append("parents: " + parentsToString() + "\n");
		sb.append("children: " + childrenToString() + "\n");
		sb.append("descriptors: " + descriptorsToString());
		return sb.toString();
	}
}
