package vavetam.implementation.structure.refo;

import static java.util.Collections.unmodifiableSet;

import java.util.HashSet;
import java.util.Set;

import vavetam.api.structure.IDescriptor;
import vavetam.api.structure.IReferentialObject;
import vavetam.implementation.generation.incremental.formulation.Dictionary.Gender;
import vavetam.implementation.structure.enums.RefOType;


public class ReferentialObjectImpl implements IReferentialObject {
	
	private final String identifier;
	private final RefOType type;
	private final Set<IReferentialObject> parents;
	private final Set<IReferentialObject> children;
	private final Set<IDescriptor> descriptors;
	private String definiteIdentification;
	private Gender gender;	

	ReferentialObjectImpl(String identifier, RefOType type) {
		this.identifier = identifier;
		this.type = type;
		this.parents = new HashSet<IReferentialObject>();
		this.children = new HashSet<IReferentialObject>();
		this.descriptors = new HashSet<IDescriptor>();
		this.definiteIdentification = null;
	}


	@Override
	public String getIdentifier() {
		return identifier;
	}

	@Override
	public RefOType getType() {
		return type;
	}
	
	@Override
	public String getDefiniteIdentification() {
		return definiteIdentification;
	}

	@Override
	public Set<IReferentialObject> getParents() {
		return unmodifiableSet(parents);
	}

	@Override
	public Set<IReferentialObject> getChildren() {
		return unmodifiableSet(children);
	}
	
	@Override 
	public Set<IDescriptor> getDescriptors() {
		return unmodifiableSet(descriptors);
	}
	
	@Override
	public Gender getGender() {
		return gender;
	}
	
	@Override
	public boolean equals(Object o) {
		if (o instanceof ReferentialObjectImpl) {
			IReferentialObject r = (IReferentialObject) o;
			return (this.getIdentifier().equals(r.getIdentifier()));
		} else {
		    return false;
		}
	}
	
	@Override
	public int hashCode() {
		return identifier.hashCode();
	}
	
	@Override
	public String toString() {
		return "Referential Object " + identifier;
	}
	
	void addChild(IReferentialObject c) {
		children.add(c);
	}
	
	void addParent(IReferentialObject p) {
		parents.add(p);
	}
	
	void setDefiniteIdentification(String defid) {
		this.definiteIdentification = defid;
	}

	void addDescriptor(IDescriptor d) {
		descriptors.add(d);
	}

	void setGender(Gender gender) {
		this.gender = gender;
	}
}
