package vavetam.oldcode;


import java.io.File;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import vavetam.api.structure.IDescriptor;
import vavetam.api.structure.IPredicate;
import vavetam.api.structure.IReferentialObject;
import vavetam.implementation.generation.incremental.formulation.Dictionary.Gender;


import static vavetam.implementation.Messages.getString;

public class XMLCoder {
	
	// TODO: Logging

	public Document toXml(Map<String, IReferentialObject> map) throws ParserConfigurationException{
		DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder docBuilder;
		docBuilder = docFactory.newDocumentBuilder();
		Document doc = docBuilder.newDocument();
		Element mapElement = doc.createElement(getString("XMLCoder_tag_map")); //$NON-NLS-1$
		mapElement.setAttribute(getString("XMLCoder_ns_vavetam"), getString("XMLCoder_ns_vavetam_loc")); //$NON-NLS-1$ //$NON-NLS-2$
		mapElement.setAttribute(getString("XMLCoder_ns_xsi"), getString("XMLCoder_ns_xsi_loc")); //$NON-NLS-1$ //$NON-NLS-2$
		mapElement.setAttribute(getString("XMLCoder_schema"), getString("XMLCoder_schema_loc")); //$NON-NLS-1$ //$NON-NLS-2$
		doc.appendChild(mapElement);
		for (IReferentialObject refo : map.values()) {
			Element refoElement = doc.createElement(getString("XMLCoder_tag_refo")); //$NON-NLS-1$
			refoElement.setAttribute(getString("XMLCoder_att_id"), refo.getIdentifier()); //$NON-NLS-1$
			refoElement.setAttribute(getString("XMLCoder_att_type"), refo.getType().toString()); //$NON-NLS-1$
			String defId = refo.getDefiniteIdentification();
			if (defId != null) {
				refoElement.setAttribute(getString("XMLCoder_att_name"), defId); //$NON-NLS-1$
			}
			Gender gend = refo.getGender();
			if (gend != null) {
				refoElement.setAttribute(getString("XMLCoder_att_gender"), gend.toString()); //$NON-NLS-1$
			}
			Set<IReferentialObject> children = refo.getChildren();
			if (!children.isEmpty()) {
				Element childrenElement = doc.createElement(getString("XMLCoder_tag_subparts")); //$NON-NLS-1$
				for (IReferentialObject child: children) {
					Element childElement = doc.createElement(getString("XMLCoder_tag_part")); //$NON-NLS-1$
					childElement.setAttribute(getString("XMLCoder_att_ref"), child.getIdentifier()); //$NON-NLS-1$
					childrenElement.appendChild(childElement);
				}
				refoElement.appendChild(childrenElement);
			}
			Set<IReferentialObject> parents = refo.getParents();
			if (!parents.isEmpty()) {
				Element parentsElement = doc.createElement(getString("XMLCoder_tag_superparts")); //$NON-NLS-1$
				for (IReferentialObject parent: parents) {
					Element parentElement = doc.createElement(getString("XMLCoder_tag_part")); //$NON-NLS-1$
					parentElement.setAttribute(getString("XMLCoder_att_ref"), parent.getIdentifier()); //$NON-NLS-1$
					parentsElement.appendChild(parentElement);
				}
				refoElement.appendChild(parentsElement);
			}
			for (IDescriptor descriptor: refo.getDescriptors()) {
				Element descriptorElement = doc.createElement(getString("XMLCoder_tag_descr")); //$NON-NLS-1$
				descriptorElement.setAttribute(getString("XMLCoder_att_defn"), descriptor.getDefiniteness().toString()); //$NON-NLS-1$
				for (IPredicate predicate: descriptor) {
					Element predicateElement = doc.createElement(getString("XMLCoder_tag_pred")); //$NON-NLS-1$
					predicateElement.setAttribute(getString("XMLCoder_att_name"), predicate.getName().toString()); //$NON-NLS-1$
					for (IReferentialObject arg : predicate.getArguments()) {
						Element argElement = doc.createElement(getString("XMLCoder_tag_arg")); //$NON-NLS-1$
						String argString = arg.getIdentifier();
						argString = argString.equals(refo.getIdentifier()) ? getString("XMLCoder_this") : argString; //$NON-NLS-1$
						argElement.setAttribute(getString("XMLCoder_att_ref"), argString); //$NON-NLS-1$
						predicateElement.appendChild(argElement);
					}
					descriptorElement.appendChild(predicateElement);
				}
				refoElement.appendChild(descriptorElement);
			}
			mapElement.appendChild(refoElement);
		}
		return doc;
	}
	
	public void writeFile(Map<String, IReferentialObject> map, String file) throws ParserConfigurationException, TransformerException {
		Document doc = toXml(map);
		TransformerFactory transformerFactory = TransformerFactory.newInstance();
		Transformer transformer;
		transformer = transformerFactory.newTransformer();
//		transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
		transformer.setOutputProperty(OutputKeys.METHOD, "xml");
		transformer.setOutputProperty(OutputKeys.INDENT, "yes");
		transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
		transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
		DOMSource source = new DOMSource(doc);
		StreamResult result = new StreamResult(new File(file));
		transformer.transform(source, result);
	}
}
