package vavetam.implementation.fileio;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static vavetam.implementation.structure.enums.Definiteness.ETA;
import static vavetam.implementation.structure.enums.PredicateName.STREET;

import java.io.File;
import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;

import vavetam.api.structure.IDescriptor;
import vavetam.api.structure.IReferentialObject;
import vavetam.implementation.fileio.XMLParser;
import vavetam.implementation.structure.util.ReferentialObjectUtils;


public class MapReadTest {
	
	Map<String, IReferentialObject> map = null;
	
	@Before
	public void setUp() {
		try {
			map = new XMLParser().parse(new File("src/main/resources/vavetam/maps/Map1Set2.xml"));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	@Test
	public void testMapGetsFilled() {
		assertTrue(map.size() > 0);
	}
	
	@Test
	public void testProperRefOs() {
		IReferentialObject pt2 = map.get("pt2");
		IReferentialObject pts23 = map.get("pts23");
		assertNotNull(pt2);
		assertNotNull(pts23);
		assertEquals("pt2", pt2.getIdentifier());
		assertEquals("pts23", pts23.getIdentifier());
	}

	@Test
	public void testRefOsAreLinked() {
		IReferentialObject pt2 = map.get("pt2");
		IReferentialObject pts23 = map.get("pts23");
		assertTrue(pt2.getChildren().contains(pts23));
		assertTrue(pts23.getParents().contains(pt2));
	}
	
	@Test
	public void testRefOsAreUnmodifiable() {
		IReferentialObject pt2 = map.get("pt2");
		IReferentialObject pts23 = map.get("pts23");
		try {
			pt2.getChildren().add(pts23);
			fail();
		} catch (Exception e) {/*succeed*/}
	}
	
	@Test
	public void testRefOsContainDescriptors() {
		IReferentialObject pt2 = map.get("pt2");
		IDescriptor d = null;
		List<IDescriptor> l = ReferentialObjectUtils.getDescriptorsContainingPredicatesWithName(STREET, pt2);
		if (l.size() > 0) {
			d = l.get(0);
		}
		assertNotNull(d);
		assertEquals(pt2, d.getPredicateAtIndex(0).getArguments().get(0));
		assertEquals(ETA, d.getDefiniteness());
		d = null;
	}
}
